/****************************************************************************
**  MDHashTool - Message Digest (MD5, SHA-1, etc.) tool
**  Copyright (c) 2005  Charles Melhorn
**
**  This file is part of MDHashTool.
**
**  MDHashTool is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  MDHashTool is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

var gTargetFile = null;

var nsIMDHash = Components.interfaces.nsIMDHash;
var gMDHashTool = Components.classes["@mozilla.org/mdhashtool;1"]
                  .createInstance(nsIMDHash);


function Init()
{
  var lf_array = null;
  var hash_type;

  var hashtype_db = document.getElementById("mdht-hashtype-dropbox"); 
  hashtype_db.value = gMDHTUserPreferenceInfo.getPrefValue("default_hash_alg");  

  if ( (window.arguments[0]) &&
       (window.arguments[0] instanceof Components.interfaces.nsIFile) )
   {
    gTargetFile = window.arguments[0];

    // check originating URL for link fingerprint and
    // override hashtype preference if found

    if (window.arguments[1])
     {
       var sourceURL = getSourceUrl(window.arguments[1]);

       if ( (sourceURL) && (lf_array = isValidLinkFingerprint(sourceURL)) )
         hashtype_db.value = lf_array[1].toUpperCase();
     }

    switch(hashtype_db.value)
    {
      case "MD5":  hash_type = nsIMDHash.HASH_MD5;
                   break;

      case "SHA1": hash_type = nsIMDHash.HASH_SHA1;
                   break;

      default:     hash_type = nsIMDHash.HASH_MD5;
                   break;
    }

    var checksum = gMDHashTool.getHashForFile(hash_type, gTargetFile);
    document.getElementById("digest1-desc").value = checksum;

    // Get IOService to convert file path to URI
    var serv = Components.classes["@mozilla.org/network/io-service;1"].
               getService(Components.interfaces.nsIIOService);

    var fileURI = serv.newFileURI(gTargetFile);
    document.getElementById("input-desc").value = fileURI.spec;

    // if link fingerprint exists, compare computed digest
    // to reference value

    if (lf_array)
     {
       compareDigests(lf_array[2]);
       document.getElementById("digest-ref-desc").value = lf_array[2];
       document.getElementById("match-indicator-desc").value += " [link fingerprint]";
       document.getElementById("digest-ref-desc").select();
     }

    document.getElementById("digest-ref-desc").focus();
   }

}


function getFileDigest(hash_alg)
{
  var hash_type, checksum;

  switch(hash_alg)
  {
    case "MD5":  hash_type = nsIMDHash.HASH_MD5;
                 break;

    case "SHA1": hash_type = nsIMDHash.HASH_SHA1;
                 break;

    default:     hash_type = nsIMDHash.HASH_MD5;
                 break;
  }

  if (gTargetFile)
   {
     checksum = gMDHashTool.getHashForFile(hash_type, gTargetFile);
   }

  document.getElementById("digest1-desc").value = checksum;

  compareDigests(document.getElementById("digest-ref-desc").value);
}


function compareDigests(digestString)
{
  if (digestString == "")
   {
    document.getElementById("match-indicator-img").setAttribute("match", "untested");
    document.getElementById("match-indicator-desc").value = "";
   }
  else
   {
     if (digestString.toLowerCase() == document.getElementById("digest1-desc").value.toLowerCase())
      {
       document.getElementById("match-indicator-img").setAttribute("match", "true");
       document.getElementById("match-indicator-desc").value = "match";
      }
     else
      {
       document.getElementById("match-indicator-img").setAttribute("match", "false");
       document.getElementById("match-indicator-desc").value = "no match";
      }
   }

}


function copyStringToClipboard(aString)
{
  var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                           .getService(Components.interfaces.nsIClipboardHelper);

  clipboardHelper.copyString(aString);  
}


function getSourceUrl(download_id)
{
  const NC_NS = "http://home.netscape.com/NC-rdf#";
  const rdfSvcContractID = "@mozilla.org/rdf/rdf-service;1";
  const rdfSvcIID = Components.interfaces.nsIRDFService;
  var rdfService = Components.classes[rdfSvcContractID].getService(rdfSvcIID);

  const dlmgrContractID = "@mozilla.org/download-manager;1";
  const dlmgrIID = Components.interfaces.nsIDownloadManager;
  var downloadMgr = Components.classes[dlmgrContractID].getService(dlmgrIID);
  var ds = downloadMgr.datasource;
  
  var resource = rdfService.GetUnicodeResource(download_id);
  var sourceRes = rdfService.GetResource(NC_NS + "URL");
  
  var source = ds.GetTarget(resource, sourceRes, true).QueryInterface(Components.interfaces.nsIRDFResource).Value;

  return source;
}


// returns array resulting from match operation
// if valid, null if not

function isValidLinkFingerprint(href)
{
  var cs_array = null;
  var bValid = false;

  if ( cs_array = href.match(/#\!(md5|sha1)!([a-f0-9]+)/) )
   {
     // check hexvalue string length for corresponding hash type

     switch (cs_array[1])  
     {
       case "md5":   bValid = (cs_array[2].length == 32);
                     break;

       case "sha1":  bValid = (cs_array[2].length == 40);
                     break;

       default:      break;
     }
   }
  
  return (bValid ? cs_array : null);
 
}


